class DiscListException(Exception):
    """
    The base class to all DiscList Error Exceptions.
    """

class Unauthorized(DiscListException):
    def __init__(self, message: str):
        """
        The Exception when you did not enter a specific data.
        """
        super().__init__(message)

class BotTokenRequired(Unauthorized):
    def __init__(self, message: str):
        """
        The Exception when you did not enter a bot token.
        """
        super().__init__("You must enter a bot token.")

class NotFound(DiscListException):
    def __init__(self, message: str):
        """
        The Exception when you did enter an invalid data.
        """
        super().__init__(message)

class InvalidToken(NotFound):
    def __init__(self):
        """
        The Exception when you did enter an invalid bot token.
        """
        super().__init__("You entered an invalid bot token.")


from typing import Union, Dict
from .errors import *

import aiohttp
import requests

class Client:
    def __init__(self,  token : str = None, sync : bool = True):
        """
        The Client of DiscList to authenticate to all endpoints.

        Parameters
        ----------
        token :class:`str`
            The token of your bot to authenticate to endpoints
            that require a token.
        sync :class:`bool`
            Whether if functions should be async or not.
            If this is ``True``, it will use aiohttp,
            else it will use requests to authenticate to
            endpoints.
        """
        self.sync = sync
        self.token = token
        self.base_url = "https://disclist.noxitb.repl.co/api"

    #def _get_adapter(self):
        #if self.sync:
            #return requests.session()
        #else:
            #return aiohttp.ClientSession()

    def post_stats(self, server_count : int) -> Union[aiohttp.ClientResponse, requests.Response]:
        """
        Updates the stats of your bot.

        Paramaters
        ----------
        server_count :class:`int`
            The number of servers that your bot
            is in.
        """
        server_count = str(server_count)

        if self.sync:
            async def _post_stats():
                async with aiohttp.ClientSession() as session:
                    async with session.post(f"{self.base_url}/bots/stats", headers={"serverCount": server_count, "Authorization": self.token}) as response:
                        if response.status == 401:
                            raise BotTokenRequired()
                        elif response.status == 404:
                            raise InvalidToken()

                        return response
        else:
            def _post_stats():
                response = requests.post(f"{self.base_url}/bots/stats", headers={"serverCount": server_count, "Authorization": self.token})
                if response.status_code == 401:
                    raise BotTokenRequired()
                elif response.status_code == 404:
                    raise InvalidToken()

                return response

        return _post_stats()

    def has_voted(self, user_id : int) -> bool:
        """
        Checks if the user has voted your
        bot or not.

        Paramaters
        ----------
        user_id :class:`int`
            The user id of the user you want
            to check.
        """
        user_id = str(user_id)

        if self.sync:
            async def _has_voted():
                async with aiohttp.ClientSession() as session:
                    async with session.get(f"{self.base_url}/check/{user_id}", headers={"Authorization": self.token}) as response:
                        if response.status == 401:
                            raise Unauthorized(f"You must enter a user id.")
                        elif response.status == 403:
                            raise BotTokenRequired()
                        elif response.status == 404:
                            raise InvalidToken()

                        js = await response.json()
                        return js['voted']
        else:
            def _has_voted():
                response = requests.get(f"{self.base_url}/check/{user_id}", headers={"Authorization": self.token})

                if response.status_code == 401:
                    raise Unauthorized(f"You must enter a user id.")
                elif response.status_code == 403:
                    raise BotTokenRequired()
                elif response.status_code == 404:
                    raise InvalidToken()

                js = response.json()
                return js['voted']

        return _has_voted()

    def search(self, bot_id : int) -> Dict[str, Union[str, list]]:
        """
        Checks if the user has voted your
        bot or not.

        Paramaters
        ----------
        bot_id :class:`int`
            The bot id to search for.
        """
        user_id = str(bot_id)

        if self.sync:
            async def _has_voted():
                async with aiohttp.ClientSession() as session:
                    async with session.get(f"{self.base_url}/bots/{bot_id}") as response:
                        if response.status_code == 404:
                            raise NotFound("You entered an invalid Bot ID.")

                        js = await response.json()
                        return js
        else:
            def _has_voted():
                response = requests.get(f"{self.base_url}/bots/{bot_id}")

                if response.status_code == 404:
                    raise NotFound("You entered an invalid Bot ID.")

                js = response.json()
                return js

        return _has_voted()


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['disclist']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=3.7.4,<4.0.0', 'requests>=2.25.1,<3.0.0']

setup_kwargs = {
    'name': 'disclist',
    'version': '0.0.1',
    'description': 'A simple asynchronous/synchronous wrapper for https://disclist.noxitb.repl.co/ Bot List.',
    'long_description': '# Guides:\nHere are guides of how to use this Python Wrapper.\n\n## Initiating your Client:\n```py\nfrom disclist import Client\n\nclient = Client(token="...", sync=True) # sync is a kwarg to set if functions need to be async or not. Defaults to True.\n```\n\n## Posting Stats:\n```py\n# If you set sync to True, it will not need to be awaited\n# Else, it needs to.\n# You can check it using `client.sync` or see the kwarg you set\n# if you are unsure.\n\n# For synchronous:\nclient.post_stats(server_count=1) # Set your server count to 1.\n\n# For asynchronous:\nawait client.post_stats(server_count=1) # Set your server count to 1.\n```\n\n## For checking if user has voted or not:\n```py\n# For synchronous:\nhas_voted = client.has_voted(user_id=1234567890)\n\n# For asynchronous:\nhas_voted = await client.has_voted(user_id=1234567890)\n\nprint(has_voted) # True/False\n```\n\n## For searching a bot using it\'s ID:\n```py\n# For synchronous:\nresult = client.search(bot_id=1234567890)\n\n# For asynchronous:\nresult = await client.search(bot_id=1234567890)\n\nprint(result) # {\'...\': ...}, Returns a dict.\n```\n\n# Where should I ask help?\nJoin our [discord](https://discord.gg/nZPaZbzYsb) to ask for help!\n\n# License:\nMIT',
    'author': 'proguy914629',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.5',
}


setup(**setup_kwargs)
